#!/system/bin/sh

abort() {
  ui_print "$1"
  rm -rf $MODPATH 2>/dev/null
  cleanup
  rm -rf $TMPDIR 2>/dev/null
  exit 1
}

cleanup() {
  rm -rf $MODPATH/common 2>/dev/null
}

device_check() {
  local opt=$(getopt -o dm -- "$@") type=device
  eval set -- "$opt"
  while true; do
    case "$1" in
      -d) type=device; shift;;
      -m) type=manufacturer; shift;;
      --) shift; break;;
      *) abort "Invalid device_check argument $1! Aborting!";;
    esac
  done

  local prop=$(echo "$1" | tr '[:upper:]' '[:lower:]')
  for i in /system /vendor /odm /product; do
    if [ -f $i/build.prop ]; then
      for j in "ro.product.$type" "ro.build.$type" "ro.product.vendor.$type" "ro.vendor.product.$type"; do
        [ "$(sed -n "s/^$j=//p" $i/build.prop 2>/dev/null | head -n 1 | tr '[:upper:]' '[:lower:]')" == "$prop" ] && return 0
      done
      [ "$type" == "device" ] && [ "$(sed -n "s/^ro.build.product=//p" $i/build.prop 2>/dev/null | head -n 1 | tr '[:upper:]' '[:lower:]')" == "$prop" ] && return 0
    fi
  done
  return 1
}

cp_ch() {
  local opt=$(getopt -o nr -- "$@") BAK=true UBAK=true FOL=false
  eval set -- "$opt"
  while true; do
    case "$1" in
      -n) UBAK=false; shift;;
      -r) FOL=true; shift;;
      --) shift; break;;
      *) abort "Invalid cp_ch argument $1! Aborting!";;
    esac
  done

  local SRC="$1" DEST="$2" OFILES="$1"
  $FOL && OFILES=$(find $SRC -type f 2>/dev/null)
  [ -z $3 ] && PERM=0644 || PERM=$3

  case "$DEST" in
    $TMPDIR/*|$MODULEROOT/*|$NVBASE/modules/$MODID/*) BAK=false;;
  esac

  for OFILE in ${OFILES}; do
    if $FOL; then
      FILE=$(echo $OFILE | sed "s|$SRC|$DEST/$(basename $SRC)|")
    else
      [ -d "$DEST" ] && FILE="$DEST/$(basename $SRC)" || FILE="$DEST"
    fi

    if $BAK && $UBAK; then
      [ ! "$(grep "$FILE$" $INFO 2>/dev/null)" ] && echo "$FILE" >> $INFO
      [ -f "$FILE" -a ! -f "$FILE~" ] && { mv -f $FILE $FILE~; echo "$FILE~" >> $INFO; }
    elif $BAK; then
      [ ! "$(grep "$FILE$" $INFO 2>/dev/null)" ] && echo "$FILE" >> $INFO
    fi

    install -D -m $PERM "$OFILE" "$FILE"
  done
}

install_script() {
  local INPATH=$NVBASE/service.d
  [ "$1" == "-l" ] && { shift; INPATH=$NVBASE/service.d; }
  [ "$1" == "-p" ] && { shift; INPATH=$NVBASE/post-fs-data.d; }

  [ "$(grep "#!/system/bin/sh" $1)" ] || sed -i "1i #!/system/bin/sh" $1

  for i in "MODPATH" "LIBDIR" "MODID" "INFO" "MODDIR"; do
    sed -i "1a $i=$(eval echo \$$i)" $1
  done

  [ "$1" == "$MODPATH/uninstall.sh" ] && return 0

  case $(basename $1) in
    post-fs-data.sh|service.sh) ;;
    *) cp_ch -n $1 $INPATH/$(basename $1) 0755;;
  esac
}

prop_process() {
  sed -i -e "/^#/d" -e "/^ *$/d" $1
  [ -f $MODPATH/system.prop ] || touch $MODPATH/system.prop
  while read LINE; do
    echo "$LINE" >> $MODPATH/system.prop
  done < $1
}

[ -z $MINAPI ] || { [ $API -lt $MINAPI ] && abort "! Your system API of $API is less than the minimum api of $MINAPI! Aborting!"; }
[ -z $MAXAPI ] || { [ $API -gt $MAXAPI ] && abort "! Your system API of $API is greater than the maximum api of $MAXAPI! Aborting!"; }
[ $API -lt 26 ] && DYNLIB=false
[ -z $DYNLIB ] && DYNLIB=false
[ -z $DEBUG ] && DEBUG=false
INFO=$NVBASE/modules/.$MODID-files
ORIGDIR="$MAGISKTMP/mirror"
LIBPATCH="\/vendor"
LIBDIR=/system/vendor

if ! $BOOTMODE; then
  echo "- Only uninstall is supported in recovery"
  echo "  Uninstalling!"
  touch $MODPATH/remove
  [ -s $INFO ] && install_script $MODPATH/uninstall.sh || rm -f $INFO $MODPATH/uninstall.sh
  cleanup
  rm -rf $NVBASE/modules_update/$MODID $TMPDIR 2>/dev/null
  exit 0
fi

if $DEBUG; then
  echo "- Debug mode"
  echo "  Module install log will include debug info"
  echo "  Be sure to save it after module install"
  set -x
fi

echo "- Extracting module files"
unzip -o "$ZIPFILE" -x 'META-INF/*' 'common/functions.sh' -d $MODPATH >&2
[ -f "$MODPATH/common/addon.tar.xz" ] && tar -xf $MODPATH/common/addon.tar.xz -C $MODPATH/common 2>/dev/null

if [ "$(ls -A $MODPATH/common/addon/*/install.sh 2>/dev/null)" ]; then
  echo " "; echo "- Running Addons -"
  for i in $MODPATH/common/addon/*/install.sh; do
    echo "  Running $(basename $(dirname $i))..."
    . $i
  done
fi

echo "- Removing old files"
if [ -f $INFO ]; then
  while read LINE; do
    if [ "$(echo -n $LINE | tail -c 1)" == "~" ]; then
      continue
    elif [ -f "$LINE~" ]; then
      mv -f $LINE~ $LINE
    else
      rm -f $LINE
      while true; do
        LINE=$(dirname $LINE)
        [ "$(ls -A $LINE 2>/dev/null)" ] && break || rm -rf $LINE
      done
    fi
  done < $INFO
  rm -f $INFO
fi

echo "- Installing"
[ -f "$MODPATH/common/install.sh" ] && . $MODPATH/common/install.sh
echo "   Installing for $ARCH SDK $API device..."

for i in $(find $MODPATH -type f -name "*.sh" -o -name "*.prop" -o -name "*.rule"); do
  [ -f $i ] && { sed -i -e "/^#/d" -e "/^ *$/d" $i; [ "$(tail -1 $i)" ] && echo "" >> $i; } || continue
  case $i in
    "$MODPATH/service.sh") install_script -l $i;;
    "$MODPATH/post-fs-data.sh") install_script -p $i;;
    "$MODPATH/uninstall.sh") 
      if [ -s $INFO ] || [ "$(head -n1 $MODPATH/uninstall.sh)" != "# Don't modify anything after this" ]; then
        install_script $MODPATH/uninstall.sh
      else
        rm -f $INFO $MODPATH/uninstall.sh
      fi;;
  esac
done

$IS64BIT || find $MODPATH/system -type d -name "lib64" -exec rm -rf {} +
[ -d "/system/priv-app" ] || mv -f $MODPATH/system/priv-app $MODPATH/system/app 2>/dev/null
[ -d "/system/xbin" ] || mv -f $MODPATH/system/xbin $MODPATH/system/bin 2>/dev/null

if $DYNLIB; then
  find $MODPATH/system/lib* -type f ! -path "*/lib*/modules/*" -exec sh -c '
    for FILE; do
      mkdir -p $(dirname $MODPATH/system/vendor/$FILE)
      mv -f $MODPATH/system/$FILE $MODPATH/system/vendor/$FILE
      [ "$(ls -A $(dirname $MODPATH/system/$FILE))